!
! Copyright (c) 2000-2013 Yoshihide Yoshimoto
! 
#include "fftgrid.h"

program lptmm
  use fft_3d
  use util
  implicit none
  integer, parameter:: nfin = 10
  real(kind=8),parameter:: p2 = 6.28318530717958648d0
  integer:: iorg(3)
  character(len=8):: magic = 'TAPP:lpt'

  character(len=8):: sign
  complex(kind=8),allocatable:: vg(:,:,:,:)
  real(kind=8),allocatable:: vr(:,:,:,:,:), ztmp(:,:,:,:)
  type (fft3_struct) :: fs
  integer:: nvx, nvy, nvz, nrx, nry, nrz, nlx, nly, nlz
  real(kind=8):: bb(3,3), aa(3,3), w
  integer:: i1, i2, i3, k1, k2, k3, is, ios
  character(len=256):: ifname, name, argv(4)
  logical:: error
  integer:: ic, i
  character(len=256):: acount
  real(kind=8):: vrmax(2), vrmin(2), vravr(2)
  integer:: nspin
  real(kind=8):: h(3),htmp,dz

  ic = 1
  call getarg(ic, argv(ic))
  if (len_trim(argv(ic)) .eq. 0) then
     write(6,*) 'invalid fname'
     stop
  end if
  ifname = argv(ic)
  ic = ic + 1
  do i = ic, ic + 2
     call getarg(i, argv(i))
  end do
  read(argv(ic),*,iostat=ios) iorg(1)
  if (ios .eq. 0) then
     error = .false.
     read(argv(ic+1),*,iostat=ios) iorg(2)
     error = error .or. (ios .ne. 0)
     read(argv(ic+2),*,iostat=ios) iorg(3)
     error = error .or. (ios .ne. 0)
     if (error) then
        write(6,*) 'invalid position of origin'
        stop
     end if
  else
     iorg(1) = 0
     iorg(2) = 0
     iorg(3) = 0
  end if
  
  open(nfin, file=ifname, status='old', action='read', form='unformatted')
  read(nfin) sign
  if (sign .ne. magic) then
     write(6,*) 'Bad magic'
     stop
  end if

  read(nfin) bb, nspin
  read(nfin) nvx, nvy, nvz

  bb = bb/p2
  w = bb(1,1)*(bb(2,2)*bb(3,3)-bb(3,2)*bb(2,3)) &
       +bb(2,1)*(bb(3,2)*bb(1,3)-bb(1,2)*bb(3,3)) &
       +bb(3,1)*(bb(1,2)*bb(2,3)-bb(2,2)*bb(1,3))
  aa(1,1)=(bb(2,2)*bb(3,3)-bb(3,2)*bb(2,3))/w
  aa(2,1)=(bb(3,2)*bb(1,3)-bb(1,2)*bb(3,3))/w
  aa(3,1)=(bb(1,2)*bb(2,3)-bb(2,2)*bb(1,3))/w
  aa(1,2)=(bb(2,3)*bb(3,1)-bb(3,3)*bb(2,1))/w
  aa(2,2)=(bb(3,3)*bb(1,1)-bb(1,3)*bb(3,1))/w
  aa(3,2)=(bb(1,3)*bb(2,1)-bb(2,3)*bb(1,1))/w
  aa(1,3)=(bb(2,1)*bb(3,2)-bb(3,1)*bb(2,2))/w
  aa(2,3)=(bb(3,1)*bb(1,2)-bb(1,1)*bb(3,2))/w
  aa(3,3)=(bb(1,1)*bb(2,2)-bb(2,1)*bb(1,2))/w
  nrx = algn235(nvx+1,1)
  nry = algn235(nvy+1,1)
  nrz = algn235(nvz+1,1)
  ! nrx = algn235(nvx,1)
  ! nry = algn235(nvy,1)
  ! nrz = algn235(nvz,1)
  nlx = FFT_PAD_NX(nrx*2)
  nly = FFT_PAD_NY(nry*2)
  nlz = FFT_PAD_NZ(nrz*2)
  allocate(vg(-nvx:nvx, -nvy:nvy, -nvz:nvz, nspin), &
       FFT_A3M_DEFSZ(vr,nlx,nly,nlz,nspin), &
       FFT_A3D_DEFSZ(ztmp,nlx,nly,nlz))
  read(nfin) vg

  call fft3_init(nrx*2,nry*2,nrz*2,nlx,nly,nlz,fs)

  vr = 0.0d0
  do is=1,nspin 
     do i3=-nvz,nvz
        do i2=-nvy,nvy
           do i1=-nvx,nvx
              k1 = modulo(i1, nrx*2) + 1
              k2 = modulo(i2, nry*2) + 1
              k3 = modulo(i3, nrz*2) + 1
              FFT_A3M_RL(vr,k1,k2,k3,is) = dble(vg(i1,i2,i3,is))
              FFT_A3M_IM(vr,k1,k2,k3,is) = aimag(vg(i1,i2,i3,is))
           end do
        end do
     end do

     call fft3_bw(fs, FFT_A3M_RL(vr,1,1,1,is), ztmp)
  end do

  call fft3_done(fs)

  h(1) = aa(2,1)*aa(3,2)-aa(3,1)*aa(2,2)
  h(2) = aa(3,1)*aa(1,2)-aa(1,1)*aa(3,2)
  h(3) = aa(1,1)*aa(2,2)-aa(2,1)*aa(1,2)
  htmp = sqrt(h(1)*h(1)+h(2)*h(2)+h(3)*h(3))
  h = h/htmp
  dz = h(1)*aa(1,3)+h(2)*aa(2,3)+h(3)*aa(3,3)
  dz = abs(dz/(nrz*2))

  do i3=1,nrz*2
     do is=1,nspin
        vrmax(is) = maxval(FFT_A3M_RL(vr,1:nrx*2,1:nry*2,i3,is))
        vrmin(is) = minval(FFT_A3M_RL(vr,1:nrx*2,1:nry*2,i3,is))
        vravr(is) = sum(FFT_A3M_RL(vr,1:nrx*2,1:nry*2,i3,is))/(4*nrx*nry)
     end do
     if (nspin .eq. 1) then
        write(6,'(1x,f9.5,4(1x,e13.6))') &
             dz*(i3-1), vrmax(1), vrmin(1), &
             vrmax(1)-vrmin(1), vravr(1)
     else
        write(6,'(1x,f9.5,8(1x,e13.6))') &
             dz*(i3-1), vrmax(1), vrmin(1), &
             vrmax(1)-vrmin(1), vravr(1), &
             vrmax(2), vrmin(2), &
             vrmax(2)-vrmin(2), vravr(2)
     end if
  end do

end program lptmm
