!
! Copyright (c) 2000-2017 Yoshihide Yoshimoto
! 
program main
  use util
  use subr_readstr
  use subr_fmtconv
  implicit none
  integer,parameter:: nfin = 10, nfout = 11
  integer:: nrx2, nry2, nrz2
  real(kind=8):: aa(3,3)
  character(len=256):: basename, ifname, ofname, argv(6)
  integer:: ios
  logical:: error
  real(kind=8),allocatable:: field(:,:,:,:)
  character(len=8),parameter:: magic = 'TAPP:rho'
  character(len=8):: sign
  integer:: ic, i
  logical:: multiq, nextq
  character(len=256):: acount
  integer:: nspin
  integer:: ix,iy,iz,is
  integer:: iorg(3)
  real(kind=8):: tmp1,tmp2,va,fac

  integer:: ikd,it
  real(kind=8):: tax
  integer:: nkd,nsi
  integer,allocatable:: nsikd(:)
  real(kind=8):: taa(3,3)
  integer,pointer:: kd(:)
  real(kind=8),pointer:: asi(:,:),cforce(:,:)
  real(kind=8),pointer:: zo(:),zn(:)

  ic = 1
  multiq = .false.
  call getarg(ic, argv(ic))
  if ( argv(ic) .eq. '-m' ) then
     multiq = .true.
     ic = ic + 1
  end if
  call getarg(ic, argv(ic))
  if (len_trim(argv(ic)) .eq. 0) then
     write(6,*) 'invalid fname'
     stop
  end if
  basename = argv(ic)
  ifname = trim(basename) // '.str'
  open(nfin, file=ifname, status='old', action='read', form='formatted')
  call read_strfile(nfin,nkd,nsi,tax,taa,kd,asi,cforce,zo,zn)
  close(nfin)

  ofname=trim(basename)//'.car2xtp'//achar(0)
  open(nfout, file=ofname)
  write(nfout, '(1x,a)') '# atom index map from CHGCAR to xTAPP'
  write(nfout, '(1x,i6,1x,i6)') nkd, nsi
  do ikd=1,nkd
     do it=1,nsi
        if (kd(it).eq.ikd) then
           write(nfout,'(1x,i6)') it
        end if
     end do
  end do
  close(nfout)

  ifname = trim(basename) // '.rho'
  open(nfin, file=ifname, status='old', action='read', form='unformatted')
  ic = ic + 1
  do i = ic, ic + 2
     call getarg(i, argv(i))
  end do
  read(argv(ic),*,iostat=ios) iorg(1)
  if (ios .eq. 0) then
     error = .false.
     read(argv(ic+1),*,iostat=ios) iorg(2)
     error = error .or. (ios .ne. 0)
     read(argv(ic+2),*,iostat=ios) iorg(3)
     error = error .or. (ios .ne. 0)
     if (error) then
        write(6,*) 'invalid position of origin'
        stop
     end if
  else
     iorg(1) = 0.0d0
     iorg(2) = 0.0d0
     iorg(3) = 0.0d0
  end if

  rewind nfin
  if (multiq) then
     ic = 1
  end if
  read(nfin) sign
  if (sign .ne. magic) then
     write(6,*) 'bad magic: ', sign
     stop
  end if
  nextq = .true.
  do while (nextq)
     read(nfin) aa, nspin
     read(nfin) nrx2, nry2, nrz2
     allocate(field(nrx2,nry2,nrz2,nspin))
     read(nfin) field

     va = cellvol(aa)
     fac = va
     
     if (multiq) then
        write(acount, '(''.'',i3.3)') ic
        ofname=trim(basename)//trim(acount)//'.chgcar'//achar(0)
     else
        ofname=trim(basename)//'.chgcar'//achar(0)
     end if

     open(nfout, file=ofname)
     write(nfout, '(1x,a)') 'CHGCAR file'
     write(nfout, '(1x,e18.11)') 1.0d0 
     write(nfout,'(3(1x,e18.11))') aa(:,1)*oneau
     write(nfout,'(3(1x,e18.11))') aa(:,2)*oneau
     write(nfout,'(3(1x,e18.11))') aa(:,3)*oneau
     do ikd=1,nkd
        write(nfout,'(1x,a)',advance='no')  atom_name(nint(zn(ikd)))
     end do
     write(nfout,*)
     allocate(nsikd(nkd))
     nsikd = 0
     do it=1,nsi
        nsikd(kd(it)) = nsikd(kd(it)) + 1
     end do
     do ikd=1,nkd
        write(nfout,'(1x,i6)',advance='no') nsikd(ikd)
     end do
     write(nfout,*)
     write(nfout,'(1x,a)') 'Direct'
     do ikd=1,nkd
        do it=1,nsi
           if (kd(it).eq.ikd) then
              write(nfout,'(1x,3f16.12)') &
                   asi(1,it),asi(2,it),asi(3,it)
           end if
        end do
     end do
     write(nfout,*)
     write(nfout,'(3(1x,i6))') nrx2,nry2,nrz2
     if (nspin.eq.2) then
        do iz=1,nrz2
           do iy=1,nry2
              do ix=1,nrx2
                 tmp1 = field(ix,iy,iz,1) + field(ix,iy,iz,2)
                 tmp2 = field(ix,iy,iz,1) - field(ix,iy,iz,2)
                 field(ix,iy,iz,1) = tmp1
                 field(ix,iy,iz,2) = tmp2
              end do
           end do
        end do
     end if
     do is=1,nspin
        write(nfout,'(5(1x,e18.11))') &
             (((fac*field(modulo(ix,nrx2)+1,modulo(iy,nry2)+1, &
             modulo(iz,nrz2)+1,is),ix=iorg(1),iorg(1)+nrx2-1), &
             iy=iorg(2),iorg(2)+nry2-1),iz=iorg(3),iorg(3)+nrz2-1)
     end do

     deallocate(nsikd)
     deallocate(field)

     close(nfout)

     if (.not. multiq) then
        exit
     end if

     read(nfin, iostat=ios) sign
     if (ios .ne. 0) then
        nextq = .false.
     else if (sign .ne. magic) then
        write(6,*) 'bad magic: ', sign
        stop
     end if
     ic = ic + 1
  end do
  stop
end program main
