!
! Copyright (c) 2000-2013,2017 Yoshihide Yoshimoto
! 
! argument: ix
!   The original unit vectors in real space a_i is
!   transformed to expanded unit vectors ax_i as
!
!   (ax_1,ax_2,ax_3) = (a_1,a_2,a_3)*ix.
!
!   All element of ix are integer.
!
!   Therefore original unit vectors in the reciprocal space b_i is
!   transformed to expanded unit vectors bx_i as
!
!   (b_1,b_2,b_3) = (bx_1,bx_2,bx_3)*(ix)^t
!
#include "fftgrid.h"

program rok2dx
  use fft_3d
  implicit none
  integer, parameter:: nfin = 10, nfout = 11

  character(len=8):: magic = 'TAPP:rok'

  integer,parameter:: iorg(3) = (/ 0, 0, 0 /)
  character(len=8):: sign
  integer:: nrx2, nry2, nrz2, nkt, nbba1, nbba2
  real(kind=8),allocatable:: rhoin(:,:,:,:), rho(:,:,:,:), rtmp(:,:,:,:)
  real(kind=8),allocatable:: tmpf(:,:,:,:), tmpb(:,:,:,:)
  real(kind=8):: aa(3,3), sk(3), ee
  integer:: ix(3,3), mx, my, mz
  integer:: ios, ik, ib, i1, i2, i3, j1, j2, j3, k1, k2, k3
  integer:: is, nspin, ncomp, idens
  character(len=256):: ifname, ofname, name, buf1, buf2, buf3, argv(5)
  logical:: expandq, error
  type(fft3_struct):: fsf, fsb

  call getarg(1, argv(1))
  if (len_trim(argv(1)) .eq. 0) then
     write(6,*) 'invalid fname'
     stop
  end if
  ifname = argv(1)
  call getarg(2, argv(2))
  call getarg(3, argv(3))
  call getarg(4, argv(4))
  call getarg(5, argv(5))
  read(argv(2),*,iostat=ios) ix(1,1), ix(2,1), ix(3,1)
  if (ios .eq. 0) then
     error = .false.
     read(argv(3),*,iostat=ios)  ix(1,2), ix(2,2), ix(3,2)
     error = error .or. (ios .ne. 0)
     read(argv(4),*,iostat=ios)  ix(1,3), ix(2,3), ix(3,3)
     error = error .or. (ios .ne. 0)
     read(argv(5),*,iostat=ios)  mx, my, mz
     error = error .or. (ios .ne. 0)
     if (error) then
        write(6,*) 'invalid position of origin'
        stop
     end if
     expandq = .true.
  else
     ix = 0
     ix(1,1) = 1
     ix(2,2) = 1
     ix(3,3) = 1
     expandq = .false.
  end if

  open(nfin, file=ifname, status='old', action='read', form='unformatted')
  read(nfin) sign
  if (sign .ne. magic) then
     write(6,*) 'Bad magic'
     stop
  end if
  read(nfin,iostat=ios) aa,nspin,ncomp
  if (ios.ne.0) then
     ncomp = 1
     backspace(nfin)
     read(nfin) aa,nspin
  end if
  if (nspin*ncomp.ne.1 .and. nspin*ncomp.ne.2) then
     write(6,*) 'invalid number_spin and number_component in rok file'
     stop
  end if
  read(nfin) nrx2,nry2,nrz2
  read(nfin) nkt,nbba1,nbba2
  if (.not. expandq) then
     mx = nrx2
     my = nry2
     mz = nrz2
  else
     call fft3_init(nrx2,nry2,nrz2,nrx2,nry2,nrz2,fsf)
     call fft3_init(mx,my,mz,mx,my,mz,fsb)
     aa = matmul(aa,ix)
  end if

  allocate(FFT_A3D_DEFSZ(rhoin,nrx2,nry2,nrz2), FFT_A3D_DEFSZ(rho,mx,my,mz))
  allocate(FFT_A3D_DEFSZ(tmpf,nrx2,nry2,nrz2), FFT_A3D_DEFSZ(tmpb,mx,my,mz))
  allocate(rtmp(mx,my,mz,ncomp**2))

  do ik=1,nkt
     read(nfin) sk
     do is=1,nspin
        do ib=0,nbba2-nbba1
           write(buf1,'(i0)') ib + nbba1
           buf1=adjustl(buf1)
           write(buf2,'(i0)') ik
           buf2=adjustl(buf2)

           write(buf3,'(i0)') is
           buf3=adjustl(buf3)

           read(nfin) ee
           do idens=1,ncomp**2
              if (expandq) then
                 read(nfin) FFT_A3D_RL(rhoin,:,:,:)
                 FFT_A3D_IM(rhoin,:,:,:) = 0.0d0

                 call fft3_fw(fsf, rhoin, tmpf)

                 rho = 0.0d0
                 do i3=-nrz2/2+1,nrz2/2-1
                    do i2=-nry2/2+1,nry2/2-1
                       do i1=-nrx2/2+1,nrx2/2-1
                          j1 = ix(1,1)*i1+ix(2,1)*i2+ix(3,1)*i3
                          j2 = ix(1,2)*i1+ix(2,2)*i2+ix(3,2)*i3
                          j3 = ix(1,3)*i1+ix(2,3)*i2+ix(3,3)*i3
                          if ((abs(j1).lt.mx/2).and.(abs(j2).lt.my/2) &
                               .and.(abs(j3).lt.mz/2)) then
                             k1 = mod(i1+nrx2,nrx2)+1
                             k2 = mod(i2+nry2,nry2)+1
                             k3 = mod(i3+nrz2,nrz2)+1
                             j1 = mod(j1+mx,mx)+1
                             j2 = mod(j2+my,my)+1
                             j3 = mod(j3+mz,mz)+1
                             FFT_A3D_RL(rho,j1,j2,j3) = FFT_A3D_RL(rhoin,k1,k2,k3)
                             FFT_A3D_IM(rho,j1,j2,j3) = FFT_A3D_IM(rhoin,k1,k2,k3)
                          end if
                       end do
                    end do
                 end do

                 call fft3_bw(fsb, rho, tmpb)
              else
                 read(nfin) FFT_A3D_RL(rho,:,:,:)
              end if
              rtmp(:,:,:,idens) = FFT_A3D_RL(rho,:,:,:)
           end do

           name='TAPP rok'//achar(0)
           ofname=trim(ifname)//'.'//trim(buf3)//'.'//trim(buf1) &
                //'.'//trim(buf2)//'.dx'//achar(0)
           call outfldprsk(ofname, name, mx, my, mz, aa, iorg, &
                ncomp**2, mx, my, mz, rtmp, sk, ee, 0)
        end do
     end do
  end do
  deallocate(rho)
end program rok2dx
