!
! Copyright (c) 2000-2013 Yoshihide Yoshimoto
! 
program main
  implicit none
  integer,parameter:: nfin = 10, nfout = 11
  real(kind=8),parameter:: oneau = 0.529177249
  character(len=*),parameter:: magic = 'stm file'
  character(len=256):: ifname, ofname, argv(4), suf
  integer:: nspin, nrx, nry, nrz
  real(kind=8):: aa(3,3)
  real(kind=4):: delta(3,3)
  real(kind=8),allocatable:: field(:,:,:,:),bias(:)
  character(len=8):: buf

  integer:: is,nbias,ibias

  integer:: ic,izst,ix,iy,iz
  real(kind=8):: dens,hgt,xx,yy,delx,dely,delz,csxy,hgtmax

  ic = 1
  call getarg(ic, argv(ic))
  ifname = trim(argv(ic))
  open(nfin, file=ifname, status='old', action='read', form='unformatted')

  ic = ic + 1
  call getarg(ic, argv(ic))
  read(argv(ic),*) dens
  write(6,*) 'dens = ', dens

  ic = ic + 1
  call getarg(ic, argv(ic))
  read(argv(ic),*) izst
  write(6,*) 'izst = ', izst

  read(nfin) buf
  if (buf .ne. magic) then
     write(7,*)'bad magic'
     stop
  endif
  read(nfin) aa, nspin
  read(nfin) nrx, nry, nrz
  allocate(field(nrx*2,nry*2,nrz*2,nspin))
  read(nfin) nbias
  allocate(bias(nbias))

  delta(:,1) = oneau*aa(:,1)/(2*nrx)
  delta(:,2) = oneau*aa(:,2)/(2*nry)
  delta(:,3) = oneau*aa(:,3)/(2*nrz)

  delx = sqrt( delta(1,1)**2+delta(2,1)**2+delta(3,1)**2 )
  dely = sqrt( delta(1,2)**2+delta(2,2)**2+delta(3,2)**2 )
  delz = sqrt( delta(1,3)**2+delta(2,3)**2+delta(3,3)**2 )
  csxy = delta(1,1)*delta(1,2)+delta(2,1)*delta(2,2)+delta(3,1)*delta(3,2)
  csxy = csxy/delx/dely

  do ibias = 1, nbias
     read(nfin) bias(ibias), field
     do is=1,nspin
        write(suf,'(''.'',i1,''.'',i0,''.cci'')') is,ibias
        ofname = trim(ifname) // suf
        open(nfout, file=ofname, status='unknown', action='write', form='formatted')

        write(nfout,'(1x,a)') '# x [Ang], y[Ang], height[Ang]'
        hgtmax = 0.0d0
        do iy=1,2*nry
           do ix=1,2*nrx

              if (field(ix,iy,min(izst,2*nrz),is).ge.dens) then
                 hgt = min(izst,2*nrz)*delz
                 write(6,*) field(ix,iy,min(izst,2*nrz),is)
              else 
                 do iz=min(izst,2*nrz)-1,1,-1
                    if (field(ix,iy,iz,is).ge.dens) then
                       exit
                    end if
                 end do
                 if (iz.gt.1) then
                    hgt = ( (dens-field(ix,iy,iz,is))*iz + (field(ix,iy,iz+1,is)-dens)*(iz-1) ) &
                         / ( field(ix,iy,iz+1,is) - field(ix,iy,iz,is) ) * delz
                 else
                    hgt = 0.0d0
                 end if
              end if
              !xx = delta(1,1)*(ix-1) + delta(1,2)*(iy-1)
              !yy = delta(2,1)*(ix-1) + delta(2,2)*(iy-1)
              xx = delx*(ix-1) + dely*(iy-1)*csxy
              yy = dely*(iy-1)*sqrt(1-csxy**2)
              write(nfout,'(3(1x,f7.3))') xx,yy,hgt
              hgtmax = max(hgtmax,hgt)
           end do
           write(nfout,*)
        end do
        write(nfout,'(1x,a,e12.5)') '# height at dens = ', dens
        write(nfout,'(1x,a,e12.5)') '# density max = ', maxval(field(:,:,:,is))
        write(nfout,'(1x,a,f7.3)') '# height max [Ang] = ', hgtmax
        write(nfout,'(1x,a,f7.4)') '# bias [eV] = ', bias(ibias)
        close(nfout)
     end do
  end do

  deallocate(field,bias)
  stop
end program main
