!
! Copyright (c) 2012-2013 Yoshihide Yoshimoto
!
program tailmdlog
  use util
  implicit none

  real(kind=8):: aa0(3,3),aa(3,3),va0
  real(kind=8):: deltm,time
  integer:: nsi,nkd
  real(kind=8),dimension(:,:),allocatable:: asi,casi,casir,cforce,cavl, &
       cforceai
  integer,allocatable:: kd(:)
  real(kind=8):: etotal,enthalpy,eknuc,sysvol,total_stress,order
  real(kind=8):: setotal(6), setotalai(6)
  real(kind=8):: etotalai,enthalpyai,total_stress_ai
  real(kind=8):: stern_cons
  real(kind=8):: totmom(3),ctomas(3)
  real(kind=8):: zeta,eta,sigma
  logical:: ifperfai
  real(kind=8):: sysscl
  real(kind=8):: mdstat(6),fzeta,feta,syskbt,syspext,settr
  integer:: nstern
  real(kind=8),allocatable:: mdastat(:,:,:)
  real(kind=8):: wlupdt
  integer:: imdstp

  integer:: nstep,istep

  integer:: it
  character(len=256):: basename, ofname, ifname, argv(6)
  character(len=16):: mdmode
  integer,parameter:: nfin = 10,nfout = 11
  real(kind=8),parameter:: one_au = 0.529177249
  integer:: ios

  if (iargc() .lt. 3 .or. iargc() .gt. 6) then
     write(6,*) 'tailmdlog fname mdmode imdstp [syskbt syspext [wlupdt] ]'
     stop
  end if

  call getarg(1, argv(1))
  if (len_trim(argv(1)) .eq. 0) then
     write(6,*) 'invalid fname'
     stop
  end if
  basename = argv(1)

  call getarg(2, argv(2))
  read(argv(2),*) mdmode
  write(6,*) 'mdmode = ', mdmode

  call getarg(3, argv(3))
  read(argv(3),*) imdstp
  write(6,*) 'imdstp for next = ', imdstp

  if ( trim(mdmode) .eq.'MBMT_ST' &
       .or. trim(mdmode) .eq. 'NPT_ST' &
       .or. trim(mdmode) .eq. 'MT_ST_R' &
       .or. trim(mdmode) .eq. 'MBMT_ST_R' &
       .or. trim(mdmode) .eq. 'NPT_ST_R') then

     call getarg(4, argv(4))
     read(argv(4),*) syskbt
     write(6,*) 'syskbt = ', syskbt

     call getarg(5, argv(5))
     read(argv(5),*) syspext
     write(6,*) 'syspext = ', syspext
  else
     syspext = 0.0d0
  end if

  if ( trim(mdmode) .eq.'MBMT_ST' &
       .or. trim(mdmode) .eq. 'MT_ST_R' &
       .or. trim(mdmode) .eq. 'MBMT_ST_R' ) then
     call getarg(6, argv(6))
     read(argv(6),*) wlupdt
     write(6,*) 'wlupdt = ', wlupdt
  else
     wlupdt = 0.0d0
  end if

  ifname = trim(basename) // '.mdlog'
  open(nfin, file=ifname, status='old', action='read', form='unformatted')

  read(nfin) deltm
  read(nfin) aa0(1,1),aa0(2,1),aa0(3,1)
  read(nfin) aa0(1,2),aa0(2,2),aa0(3,2)
  read(nfin) aa0(1,3),aa0(2,3),aa0(3,3)
  read(nfin) nsi,nkd

  va0 = cellvol(aa0)

  allocate(asi(3,nsi),casi(3,nsi),casir(3,nsi),cforce(3,nsi),cavl(3,nsi), &
       kd(nsi),cforceai(3,nsi))

  istep = 0
  nstep = 0
  do while (.true.)
     read(nfin,IOSTAT=ios) ifperfai,time
     if (ios.lt.0) exit
     istep = istep + 1
     if (ifperfai) then
        read(nfin) etotal, etotalai, order
        read(nfin) enthalpy, enthalpyai
     else
        read(nfin) etotal, order
        read(nfin) enthalpy
     end if
     read(nfin) eknuc
     read(nfin) sysvol
     read(nfin) stern_cons
     read(nfin) totmom, ctomas
     read(nfin) zeta,eta,sigma
     read(nfin) kd,casi
     if (ifperfai) then
        read(nfin) total_stress, total_stress_ai, setotal, setotalai
        read(nfin) cforce, cforceai
     else
        read(nfin) total_stress, setotal
        read(nfin) cforce
     end if
     read(nfin) cavl

     nstep = nstep + 1

     sysscl = (sysvol/va0)**(1.0d0/3.0d0)
     aa = aa0*sysscl

     call cart2cellr(aa,nsi,casi,asi)
     call cell2cart(aa,nsi,asi,casir)

  end do

  ofname = trim(basename) // '.mdtail'
  open(nfout, file=ofname, status='unknown', action='write', form='formatted')

  write(nfout,'(a,e18.11)') ' # time = ', time
  write(nfout,'(a,e18.11)') ' # etotal = ', etotal
  write(nfout,'(a,e18.11)') ' # sysvol = ', sysvol
  write(nfout,'(a,e18.11)') ' # enthalpy = ', enthalpy
  write(nfout,'(a,e18.11)') ' # eknuc = ', eknuc
  write(nfout,'(a,e18.11)') ' # total_stress = ', total_stress
  write(nfout,'(a,6(1x,e18.11))') ' # setotal =', setotal(1:6)
  write(nfout,'(a,e18.11)') ' # order = ', order
  write(nfout,'(a,1x,i6,1x,i3)') ' # nsi, nkd = ', nsi, nkd
  write(nfout,*)

  write(nfout,'(1x,3(1x,e18.11),a)') aa0(1,1),aa0(2,1),aa0(3,1), &
       ' # aa0(1,1), aa0(2,1), aa0(3,1)'
  write(nfout,'(1x,3(1x,e18.11))') aa0(1,2),aa0(2,2),aa0(3,2)
  write(nfout,'(1x,3(1x,e18.11))') aa0(1,3),aa0(2,3),aa0(3,3)
  write(nfout,*)

  write(nfout,'(1x,i3,3(1x,e18.11),a)') kd(1), &
       asi(1,1),asi(2,1),asi(3,1), ' # kd, asi(1), asi(2), asi(3)'
  do it=2,nsi
     write(nfout,'(1x,i3,3(1x,e18.11))') kd(it), &
          asi(1,it),asi(2,it),asi(3,it)
  end do
  write(nfout,*)

  write(nfout,'(5(1x,e18.11),a)') sysvol, zeta, eta, sigma, stern_cons, &
       ' # sysvol, zeta, eta, sigma, stern_cos'
  write(nfout,*)

  write(nfout,'(3(1x,e18.11),a)') cavl(1,1),cavl(2,1),cavl(3,1), &
       ' # cavl(1), cavl(2), cavl(3)'
  do it=2,nsi
     write(nfout,'(3(1x,e18.11))') cavl(1,it),cavl(2,it),cavl(3,it)
  end do
  close(nfout)

  ofname = trim(basename) // '.tail.mdcnt'
  open(nfout, file=ofname, status='unknown', action='write', form='unformatted')

  mdstat(1) = zeta
  mdstat(2) = eta
  mdstat(3) = sigma

  nstern = 3*nsi - 3
  settr = setotal(1)+setotal(2)+setotal(3)

  allocate(mdastat(3,nsi,1))

  if (trim(mdmode) .eq. 'NVE' .or. trim(mdmode) .eq. 'NVT_BRDSN') then
     mdstat(4:6) = 0.0d0

     mdastat(:,:,1) = cforce
  else if (trim(mdmode) .eq.'MBMT_ST' &
       .or. trim(mdmode) .eq. 'NPT_ST' &
       .or. trim(mdmode) .eq. 'MT_ST_R' &
       .or. trim(mdmode) .eq. 'MBMT_ST_R' &
       .or. trim(mdmode) .eq. 'NPT_ST_R') then
     !
     ! CAUTION: In mdnpt_st(), eknuc is divided by 2
     !          after the calculation of fzeta.
     !
     fzeta = (2.0d0*eknuc)/(nstern*syskbt) - 1.0d0
     feta = ( settr - 3*syspext*sysvol )/( nstern*syskbt )

     mdstat(4) = fzeta
     mdstat(5) = feta
     mdstat(6) = wlupdt

     mdastat(:,:,1) = cforce
  end if

  write(nfout) len_trim(mdmode)
  write(nfout) trim(mdmode)
  write(nfout) casi, cavl, sysvol
  write(nfout) imdstp
  write(nfout) mdstat, mdastat

  close(nfout)

  deallocate(mdastat)

  stop
end program tailmdlog
