!
! Copyright (c) 2000-2013 Yoshihide Yoshimoto
!
program ltzpdos
  implicit none
  integer,parameter:: nfin = 10, nfout = 11
  real(kind=8),allocatable:: sk(:,:),ovlp(:,:,:,:),ee(:,:,:)
  integer:: nks,nw,ifbunp,nspin,nbba1,nbba2,itwav,wavil
  integer:: nkmesh
  real(kind=8),allocatable:: bzwk(:)
  integer:: ib,iks,is,ic,i,im
  character(len=256):: argv(6), ifname, ofname
  integer:: neng
  real(kind=8):: emin, emax, eng, epseng
  real(kind=8):: fac,tdos(2)
  real(kind=8),allocatable:: pdos(:,:), asta(:,:)
  real(kind=8),parameter:: pi = 3.14159265358979323846d0
  real(kind=8):: ef
  logical:: iffilsta

  integer:: idum,ios

  call getarg(1, argv(1))
  ifname = argv(1)
  open(nfin, file=ifname, status='old', action='read', form='formatted')
  ofname = trim(ifname)//'.pdos'
  open(nfout, file=ofname, status='unknown', action='write', form='formatted')
  call getarg(2, argv(2))
  read(argv(2),*) neng
  call getarg(3, argv(3))
  read(argv(3),*) emin
  call getarg(4, argv(4))
  read(argv(4),*) emax
  call getarg(5, argv(5))
  read(argv(5),*) epseng
  
  call getarg(6, argv(6))
  read(argv(6),*,iostat=ios) ef
  if (ios .eq. 0) then
     iffilsta = .true.
  else
     iffilsta = .false.
  end if

  write(nfout,'(1x,a,e18.11)') '# epseng = ', epseng
  
  read(nfin,*) nks, nw, ifbunp, nspin, nbba1, nbba2, nkmesh
  write(nfout,'(1x,a,i4)') '# ifbunp = ', ifbunp
  if (ifbunp.ne.3) then
     write(6,'(1x,a)') 'ifbunp != 3'
     stop
  end if
  if (nspin.eq.1) then
     write(nfout,'(1x,a)') '# nspin = 1'
  else if (nspin.eq.2) then
     write(nfout,'(1x,a)') '# nspin = 2'
  else
     write(6,'(1x,a)') 'invalid value in nspin'
     stop
  end if
  read(nfin,*) itwav, wavil
  write(nfout,'(1x,a,i5,1x,i5)') '# itwav, wavil = ', itwav, wavil
  if (iffilsta) then
     write(nfout,'(1x,a,e14.7)') '# filled state only. ef = ',ef
  end if

  allocate(sk(3,nks),bzwk(nks))
  allocate(ee(nw,nks,nspin),ovlp(2*wavil-1,nw,nks,nspin))
  allocate(pdos(2*wavil-1,nspin),asta(2*wavil-1,nspin))

  do is=1,nspin
     do iks=1,nks
        read(nfin,*) idum, sk(:,iks), bzwk(iks)
        if (idum.ne.iks) then
           write(6,'(1x,a)') 'idum != iks'
           stop
        end if
        do ib=1,nw
           read(nfin,*) ee(ib,iks,is), &
                (ovlp(im,ib,iks,is),im=1,2*wavil-1)
        end do
     end do
  end do

  asta = 0.0d0
  do i=0,neng
     eng = emin + (emax-emin)/neng*i
     tdos = 0.0d0
     pdos = 0.0d0
     do is=1,nspin
        do iks=1,nks
           do ib=1,nw
              fac = epseng &
                   /( pi*(epseng**2 + (eng - ee(ib,iks,is))**2) )
              if (iffilsta .and. ee(ib,iks,is).gt.ef) then
                 fac = 0.0d0
              end if
              tdos(is) = tdos(is) + bzwk(iks)*fac
              do im=1,2*wavil-1
                 pdos(im,is) = pdos(im,is) + ovlp(im,ib,iks,is)*fac
              end do
           end do
        end do
     end do
     asta = asta + pdos
     if (nspin.eq.1) then
        write(nfout,'(100(1x,e14.7))') eng, 2.0d0*tdos(1)/nkmesh/nspin, &
             (2.0d0*pdos(im,1)/nkmesh,im=1,2*wavil-1)
     else
        write(nfout,'(100(1x,e14.7))') eng, &
             tdos(1)/nkmesh, (pdos(im,1)/nkmesh,im=1,2*wavil-1), &
             tdos(2)/nkmesh, (pdos(im,2)/nkmesh,im=1,2*wavil-1)
     end if
  end do
  fac = (emax-emin)/neng/nkmesh
  write(nfout,'(1x,a)') '# integration of the pdos'
  if (nspin.eq.1) then
     write(nfout,'(1x,''#'',100(1x,e14.7))') &
          (asta(im,1)*fac,im=1,2*wavil-1)
  else
     write(nfout,'(1x,''#'',100(1x,e14.7))') &
          (asta(im,1)*fac,im=1,2*wavil-1), &
          (asta(im,2)*fac,im=1,2*wavil-1)
  end if

end program ltzpdos
